<?php

//////////////////////////////////////////////////////////////////////////////
// SQL database backend

/**
 * Implements a Drupal-specific SQL database backend.
 */
class Bitcache_SQLRepository extends Bitcache_Repository implements Iterator {
  public $table;

  function __construct(array $options = array()) {
    $this->name = !empty($options['name']) ? $options['name'] : NULL;
    $this->table = !empty($this->name) ? BITCACHE_TABLE_PREFIX . $this->name : BITCACHE_TABLE_DEFAULT;
    $this->table = !empty($options['table']) ? $options['table'] : $this->table;
    $this->table = db_escape_table($this->table);
    $this->options = array(); // TODO
    $this->options = array_merge($this->options, $options);
    $this->create();
  }

  function create() {
    if (!db_table_exists($this->table)) {
      $schema = bitcache_get_schema(BITCACHE_TABLE_DEFAULT);
      $result = db_create_table($this->table, $schema);
      bitcache_get_schema($this->table, TRUE); // clear the schema cache
      return $result;
    }
    return TRUE;
  }

  function rename($old_name, $new_name, array $options = array()) {
    $old_table = $this->table; // FIXME
    $new_table = db_escape_table(BITCACHE_TABLE_PREFIX . $new_name);
    return db_rename_table($old_table, $new_table);
  }

  function destroy() {
    return db_drop_table($this->table);
  }

  // Iterator interface

  function rewind() {
    $query = db_select($this->table, $this->table);
    $query->fields($this->table, array('id', 'data'));
    $query->addExpression('LENGTH(data)', 'size');
    $query->orderBy('id', 'ASC');
    $this->query = $query->execute();

    $this->row = NULL;
    $this->next();
  }

  function key() {
    return $this->row->id;
  }

  function current() {
    return new Bitcache_Stream($this->row->id, $this->row->data, array('size' => (int)$this->row->size));
  }

  function next() {
    if (!empty($this->query)) {
      $this->row = $this->query->fetchObject();
    }
  }

  function valid() {
    return !empty($this->query) && $this->row;
  }

  // Countable interface

  function count() {
    return (int) db_query("SELECT COUNT(id) FROM {{$this->table}}")->fetchField();
  }

  // Bitcache_Repository interface

  function size() {
    return (int) db_query("SELECT SUM(LENGTH(data)) AS size FROM {{$this->table}}")->fetchField();
  }

  function exists($id) {
    return (bool) db_query("SELECT COUNT(id) AS size FROM {{$this->table}} WHERE id = :id", array(':id' => $id))->fetchField();
  }

  function get($id) {
    $query = db_select($this->table, $this->table);
    $query->fields($this->table, array('data'));
    $query->addExpression('LENGTH(data)', 'size');
    $query->condition('id', $id);
    $result = $query->execute();

    if ($row = $result->fetchObject()) {
      return new Bitcache_Stream($id, $row->data, array('size' => (int)$row->size));
    }
  }

  function put($id, $data) {
    $id = !$id ? bitcache_id($data) : $id;
    $inserted = (bool) db_insert($this->table)
      ->fields(array('id' => $id, 'data' => $data))
      ->execute();
      
    if (!$this->exists($id) && ($id = ($inserted ? $id : FALSE))) {
      $this->created($id, $data);
    }
    return $id;
  }

  function delete($id) {
    $result = db_delete($this->table)->condition('id', $id)->execute();
    if ($result > 0) { // FALSE would mean something went wrong
      $this->deleted($id);
    }
    return $result;
  }
}
